==> XFree86-4.5.0.tar.bz2: xc/programs/Xserver/hw/tinyx/ts300/ts300.c <==
/*
 * Copyright  1999 Keith Packard
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Keith Packard not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  Keith Packard makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * KEITH PACKARD DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL KEITH PACKARD BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */
/* $XFree86: xc/programs/Xserver/hw/tinyx/ts300/ts300.c,v 1.1 2004/06/02 22:43:03 dawes Exp $ */
/*
 * Copyright (c) 2004 by The XFree86 Project, Inc.
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject
 * to the following conditions:
 *
 *   1.  Redistributions of source code must retain the above copyright
 *       notice, this list of conditions, and the following disclaimer.
 *
 *   2.  Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer
 *       in the documentation and/or other materials provided with the
 *       distribution, and in the same place and form as other copyright,
 *       license and disclaimer information.
 *
 *   3.  The end-user documentation included with the redistribution,
 *       if any, must include the following acknowledgment: "This product
 *       includes software developed by The XFree86 Project, Inc
 *       (http://www.xfree86.org/) and its contributors", in the same
 *       place and form as other third-party acknowledgments.  Alternately,
 *       this acknowledgment may appear in the software itself, in the
 *       same form and location as other such third-party acknowledgments.
 *
 *   4.  Except as contained in this notice, the name of The XFree86
 *       Project, Inc shall not be used in advertising or otherwise to
 *       promote the sale, use or other dealings in this Software without
 *       prior written authorization from The XFree86 Project, Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE XFREE86 PROJECT, INC OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "tinyx.h"

extern KdCardFuncs  sisFuncs;
extern KdCardFuncs  s3Funcs;

/*
 * Kludgy code to parse the ascii /proc/pci file as the TS300
 * is running a 2.0 kernel
 */
static BOOL
HasPCI (char *name, KdCardAttr *attr)
{
    FILE    *f;
    char    line[1024];
    BOOL    waiting;
    BOOL    found = FALSE;
    char    *mem;

    f = fopen ("/proc/pci", "r");
    if (!f)
	return FALSE;
    waiting = FALSE;
    attr->naddr = 0;
    while (fgets (line, sizeof (line), f))
    {
	if (waiting)
	{
	    
	    if ((mem = strstr (line, "memory at ")))
	    {
		mem += strlen ("memory at ");
		attr->address[attr->naddr++] = strtoul (mem, NULL, 0);
		found = TRUE;
	    }
	    else if ((mem = strstr (line, "I/O at ")))
	    {
		mem += strlen ("I/O at ");
		attr->io = strtoul (mem, NULL, 0);
		found = TRUE;
	    }
	    else if (strstr (line, "Bus") && strstr (line, "device") &&
		strstr (line, "function"))
		break;
	}
	else if (strstr (line, "VGA compatible controller"))
	{
	    if (strstr (line, name))
		waiting = TRUE;
	}
    }
    fclose (f);
    return found;
}

typedef struct _PCICard {
    char	*user;
    char	*name;
    KdCardFuncs	*funcs;
} PCICard;

PCICard	PCICards[] = {
    {"sis",  "Silicon Integrated Systems",   &sisFuncs},
    {"s3",   "S3 Inc.",			    &s3Funcs},
};

#define NUM_PCI_CARDS	(sizeof (PCICards) / sizeof (PCICards[0]))
    
void
InitCard (char *name)
{
    int		i;
    KdCardAttr	attr;
    
    for (i = 0; i < NUM_PCI_CARDS; i++)
    {
	if (!name || !strcmp (name, PCICards[i].user))
	{
	    if (HasPCI (PCICards[i].name, &attr))
	    {
		KdCardInfoAdd (PCICards[i].funcs, &attr, 0);
		return;
	    }
	}
    }
}

void
InitOutput (ScreenInfo *pScreenInfo, int argc, char **argv)
{
    KdInitOutput (pScreenInfo, argc, argv);
}

void
InitInput (int argc, char **argv)
{
    KdInitInput (&LinuxMouseFuncs, &LinuxKeyboardFuncs);
}

int
ddxProcessArgument (int argc, char **argv, int i)
{
    return KdProcessArgument (argc, argv, i);
}

void
ddxUseMsg ()
{
    KdUseMsg ();
}

==> XFree86-4.7.99.12.tar.bz2: xc/programs/Xserver/hw/tinyx/ts300/ts300.c <==
/*
 * Copyright  1999 Keith Packard
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Keith Packard not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  Keith Packard makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * KEITH PACKARD DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL KEITH PACKARD BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */
/* $XFree86: xc/programs/Xserver/hw/tinyx/ts300/ts300.c,v 1.3 2007/01/23 18:02:58 tsi Exp $ */
/*
 * Copyright (c) 2004 by The XFree86 Project, Inc.
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject
 * to the following conditions:
 *
 *   1.  Redistributions of source code must retain the above copyright
 *       notice, this list of conditions, and the following disclaimer.
 *
 *   2.  Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer
 *       in the documentation and/or other materials provided with the
 *       distribution, and in the same place and form as other copyright,
 *       license and disclaimer information.
 *
 *   3.  The end-user documentation included with the redistribution,
 *       if any, must include the following acknowledgment: "This product
 *       includes software developed by The XFree86 Project, Inc
 *       (http://www.xfree86.org/) and its contributors", in the same
 *       place and form as other third-party acknowledgments.  Alternately,
 *       this acknowledgment may appear in the software itself, in the
 *       same form and location as other such third-party acknowledgments.
 *
 *   4.  Except as contained in this notice, the name of The XFree86
 *       Project, Inc shall not be used in advertising or otherwise to
 *       promote the sale, use or other dealings in this Software without
 *       prior written authorization from The XFree86 Project, Inc.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE XFREE86 PROJECT, INC OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "tinyx.h"

extern KdCardFuncs  sisFuncs;
extern KdCardFuncs  s3Funcs;

/*
 * Kludgy code to parse the ascii /proc/pci file as the TS300
 * is running a 2.0 kernel
 */
static BOOL
HasPCI (char *name, KdCardAttr *attr)
{
    FILE    *f;
    char    line[1024];
    BOOL    waiting;
    BOOL    found = FALSE;
    char    *mem;

    f = fopen ("/proc/pci", "r");
    if (!f)
	return FALSE;
    waiting = FALSE;
    attr->naddr = 0;
    while (fgets (line, sizeof (line), f))
    {
	if (waiting)
	{
	    
	    if ((mem = strstr (line, "memory at ")))
	    {
		mem += strlen ("memory at ");
		attr->address[attr->naddr++] = strtoul (mem, NULL, 0);
		found = TRUE;
	    }
	    else if ((mem = strstr (line, "I/O at ")))
	    {
		mem += strlen ("I/O at ");
		attr->io = strtoul (mem, NULL, 0);
		found = TRUE;
	    }
	    else if (strstr (line, "Bus") && strstr (line, "device") &&
		strstr (line, "function"))
		break;
	}
	else if (strstr (line, "VGA compatible controller"))
	{
	    if (strstr (line, name))
		waiting = TRUE;
	}
    }
    fclose (f);
    return found;
}

typedef struct _PCICard {
    char	*user;
    char	*name;
    KdCardFuncs	*funcs;
} PCICard;

PCICard	PCICards[] = {
    {"sis",  "Silicon Integrated Systems",   &sisFuncs},
    {"s3",   "S3 Inc.",			    &s3Funcs},
};

#define NUM_PCI_CARDS	(sizeof (PCICards) / sizeof (PCICards[0]))
    
void
InitCard (const char *name)
{
    int		i;
    KdCardAttr	attr;
    
    for (i = 0; i < NUM_PCI_CARDS; i++)
    {
	if (!name || !strcmp (name, PCICards[i].user))
	{
	    if (HasPCI (PCICards[i].name, &attr))
	    {
		KdCardInfoAdd (PCICards[i].funcs, &attr, 0);
		return;
	    }
	}
    }
}

void
InitOutput (ScreenInfo *pScreenInfo, const int argc, const char **argv)
{
    KdInitOutput (pScreenInfo, argc, argv);
}

void
InitInput (const int argc, const char **argv)
{
    KdInitInput (&LinuxMouseFuncs, &LinuxKeyboardFuncs);
}

int
ddxProcessArgument (int argc, const char **argv, int i)
{
    return KdProcessArgument (argc, argv, i);
}

void
ddxUseMsg ()
{
    KdUseMsg ();
}
